using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;
// the novapiLib package must be added as a	COM	refference
using novapiLib90;
using Globals;

namespace Hello_World_CSharp
{
    ///	<summary>
    ///	Summary	description	for	Class1.
    ///	</summary>
    class Class1
    {
        ///	<summary>
        ///	The	main entry point for the application.
        ///	</summary>
		public static string PRINTER_NAME = "novaPDF SDK 9";
        public static string PROFILE_NAME = "HelloWorld";
        public static int PROFILE_IS_PUBLIC = 0;

        public static string PDF_TEXT = "Hello World";
        public static string PDF_FILE_NAME = "HelloWorld.pdf";

        [STAThread]
        static void Main(string[] args)
        {
            try
            {
                // create the NovaPdfOptions object
                NovaPdfOptions90 pNova = new NovaPdfOptions90();
                // initialize the NovaPdfOptions object
                // if you have an application license for novaPDF SDK, 
                // pass the license key to the Initialize() function
                pNova.Initialize(PRINTER_NAME, "");

                // get the active profile ...
                string activeProfile = null;
                try
                {
                    pNova.GetActiveProfile(out activeProfile);
                }
                catch (System.Runtime.InteropServices.COMException e)
                {
                    // ignore profile exists error
                    if (NovaErrors.NV_NO_ACTIVE_PROFILE == (uint)e.ErrorCode)
                    {
                        System.Console.WriteLine("The printer does not have an active profile");
                    }
                    else
                    {
                        // more	serious	error, propagate it
                        throw e;
                    }
                }

                string _newProfileID = String.Empty;
                //add a new profile
                pNova.AddProfile(PROFILE_NAME, PROFILE_IS_PUBLIC, out _newProfileID);

                //load the new profile
                pNova.LoadProfile(_newProfileID);

                //DO NOT OPEN PDF 
                //an Open action is added by default in the profile - delete or disable this open action
                //pNova.DisableActionType((int)ActionType.NOVA_ACTION_OPEN);
           
                // and set some	options
                // uncomment the function calls for the options you wish to set
                // change the options in the nova.cs unit
                //nova.NovaTools.AddProfileOptions(pNova);
                //nova.NovaTools.AddSaveOptions(pNova);
                //nova.NovaTools.AddMergeOptions(pNova);
                //nova.NovaTools.AddDocumentInformation(pNova);
                //nova.NovaTools.AddDocumentXMPMetadata(pNova);
                //nova.NovaTools.AddViewerOptions(pNova);
                //nova.NovaTools.AddLinksOptions(pNova);
                //nova.NovaTools.AddAdvancedOptions(pNova);
                //nova.NovaTools.AddGraphicsOptions(pNova);
                //nova.NovaTools.AddEmbedFontsOptions(pNova);
                //nova.NovaTools.AddBookmarksDefinitions(pNova);
                //nova.NovaTools.AddSecurityOptions(pNova);
                //nova.NovaTools.AddWatermarkImage(pNova);
                //nova.NovaTools.AddWatermarkText(pNova);
                //nova.NovaTools.AddPageContentOptions(pNova);
                //nova.NovaTools.AddOverlayOptions(pNova);
                //nova.NovaTools.AddSignatureOptions(pNova);
                //nova.NovaTools.AddActionsOptions(pNova);

                //save the new added profile
                pNova.SaveProfile();
                //set the new profile as the active profile
                pNova.SetActiveProfile(_newProfileID);

                pNova.RegisterNovaEvent(NovaEvents.NOVAPDF_EVENT_ACTIONS);

                // print a test	page, using	the	previously set active profile settings
                using (PrintDocument pd = new PrintDocument())
                {
                    pd.PrinterSettings.PrinterName = PRINTER_NAME;
                    pd.PrintPage += new PrintPageEventHandler(PrintPageFunction);
                    pd.DefaultPageSettings.PaperSize = GetPaperSize("A4");
                    pd.Print();
                }

                int bTimeout;
                pNova.WaitForNovaEvent(120000, out bTimeout);

                pNova.SetActiveProfile(activeProfile);
                pNova.DeleteProfile(_newProfileID);
            }
            catch (System.Runtime.InteropServices.COMException e)
            {
                MessageBox.Show(e.Message);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }
        }

        //find a predefined paper size
        private static PaperSize GetPaperSize(string Name)
        {
            PaperSize size1 = null;
            Name = Name.ToUpper();
            PrinterSettings settings = new PrinterSettings();
            foreach (PaperSize size in settings.PaperSizes)
                if (size.Kind.ToString().ToUpper() == Name)
                {
                    size1 = size;
                    break;
                }
            return size1;
        }

        // and finally the function that actually prints the page
        private static void PrintPageFunction(object sender, PrintPageEventArgs ev)
        {
            string str = "novaPDF says Hello World from	C#";
            Font font = new Font("Arial", 16);
            Brush brush = new SolidBrush(Color.Black);
            ev.Graphics.DrawString(str, font, brush, 20.0f, 20.0f);
            ev.HasMorePages = false;
        }
    }
}
